/**
 * Example1 - shows JAXP/XPath/DOM use for IGK CreditReport.xml processing
 *
 * 12.03.2013 ag
 */
package igk_credit_report_jaxp_examples;

import java.io.File;
import java.io.FileInputStream;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class Example1
{

  void parseXml(InputSource isrc) throws Exception {
    try {
      DocumentBuilderFactory factory_ = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder_ = factory_.newDocumentBuilder();
      Document doc_ = builder_.parse(isrc);

      XPathFactory xPathfactory_ = XPathFactory.newInstance();

      printSubjectCompany(doc_, xPathfactory_);

    } catch (Exception x) {
      x.printStackTrace(System.out);
      throw x;
    }
  }

  static final String
  PATH_ROOT = "/document/group[@type='cr:Company Data']",
  PATH_SUMMARY = PATH_ROOT+"/group[@type='cr:Summary']",
  PATH_NAME = PATH_SUMMARY+"/item[@type='name']",
  PATH_IGKNO = PATH_SUMMARY+"/item[@type='igkno']",
  PATH_PRODUCT = PATH_SUMMARY+"/item[@type='ProductCode']",
  PATH_REVISED = PATH_SUMMARY+"/item[@type='revised']",

  PATH_SHOLD_TOPIC = PATH_ROOT+"/group[@type='cr:Shareholders']",
  PATH_SHOLD_ROWS  = PATH_SHOLD_TOPIC+"/group[@type='cr:Shareholders.Row' and not(item[@type='row.cancelled'])]";

  void printSubjectCompany(Document document, XPathFactory xPathfactory) throws Exception {
    XPath xpath_ = xPathfactory.newXPath();
log("Company   : "+xpath_.compile(PATH_NAME).evaluate(document, XPathConstants.STRING));
log("IGK number: "+xpath_.compile(PATH_IGKNO).evaluate(document, XPathConstants.STRING));
log("Product   : "+xpath_.compile(PATH_PRODUCT).evaluate(document, XPathConstants.STRING));
log("LastUpdate: "+xpath_.compile(PATH_REVISED).evaluate(document, XPathConstants.STRING));
log("");
log("Shareholders:\n"+stringShareholders(document, xPathfactory));
  }

  String stringShareholders(Document document, XPathFactory xPathfactory) throws Exception {
    String ret_ = ""; String str_;
    NodeList nlist_; Node node_;
    XPath xpath_ = xPathfactory.newXPath();
    XPathExpression expr_ = xpath_.compile(PATH_SHOLD_TOPIC);
    nlist_ = (NodeList)expr_.evaluate(document, XPathConstants.NODESET);
    if ( nlist_ != null && nlist_.getLength() > 0 ) {
      node_ = nlist_.item(0);
      Map topicComments_ = Util.getNodeChildValues(node_);
      str_ = Util.getComments(topicComments_, "top");
      if ( str_ != null ) ret_ += str_; // ends with LF

      expr_ = xpath_.compile(PATH_SHOLD_ROWS);
      nlist_ = (NodeList)expr_.evaluate(document, XPathConstants.NODESET);
      if ( nlist_ != null && nlist_.getLength() > 0 ) {
        for ( int i = 0; i < nlist_.getLength(); i++ ) {
          str_ = Util.getSholdPresentation(nlist_.item(i));
          if ( str_ != null ) ret_ += str_; // ends with LF
        }
      }

      str_ = Util.getComments(topicComments_, "bottom");
      if ( str_ != null ) ret_ += str_; // ends with LF
    }

    if ( ret_.length() == 0 ) ret_ = null;
    return ret_;
  }

  void log(String str) {
System.out.println(str);
  }

  public static void main(String[] args) {
    Example1 this_ = new Example1();
    try {
      File file_ = new File("igk_credit_report_sample.xml");
      InputSource isrc_ = new InputSource(new FileInputStream(file_));
      this_.parseXml(isrc_);
    } catch (Exception x) {
      x.printStackTrace(System.out);
    }
  }
}
