package igk_credit_report_jaxp_examples;

import java.util.HashMap;
import java.util.Map;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class Util
{
  static final String LF = new String(new char[] {10});

  public static Map<String, String> getNodeChildValues(Node node) {
//  String method_ = _class+".getNodeChildValues()";
    Map<String, String> ret_ = null;
    if ( node.hasChildNodes() ) {
      ret_ = getNodeListValues(node.getChildNodes());
    }
    return ret_;
  }

  public static Map<String, String> getNodeListValues(NodeList nlist) {
//  String method_ = _class+".getNodeListValues()";

    HashMap<String, String> map_ = null;
    if ( nlist != null ) {
      Node node_, attr_;
      String type_, name_, value_, attrName_;
      String topComments_ = "", bottomComments_ = "";
      boolean doTopComments_ = true;
      NamedNodeMap attrs_;
      map_ = new HashMap();

      for ( int i = 0; i < nlist.getLength(); i++ ) {
        node_ = nlist.item(i);
        attrs_ = node_.getAttributes();
        if ( attrs_ != null ) {
          type_ = getNodeValueAsString(attrs_.getNamedItem("type"));
          name_ = node_.getNodeName();
          if ( "group".equals(name_) ) {
            if ( "cr:TopicComments.Row".equals(type_) || "cr:RowComments.Row".equals(type_) ) {
              value_ = (String)getNodeChildValues(node_).get("comm");
              if ( doTopComments_ ) {
                topComments_ += value_+LF;
              } else {
                bottomComments_ += value_+LF;
              }
            } else {  // meaning "cr:<topic>.Row" - not parsed w. TopicComments
              doTopComments_ = false;
            }
          } else
          if ( "item".equals(name_) ) {
            doTopComments_ = false;
            if ( isLeafNode(node_) ) {
              value_ = getNodeValueAsString(node_.getFirstChild());
              if ( value_ != null ) {
                map_.put(type_, value_);
              }
            }
            // optional other attributes:
            for ( int j = 0; j < attrs_.getLength(); j++ ) {
              attr_ = attrs_.item(j);
              attrName_ = attr_.getNodeName();
              if ( !"type".equals(attrName_) ) {
                value_ = getNodeValueAsString(attr_);
                if ( value_ != null ) {
                  attrName_ = type_+"."+attrName_;
                  map_.put(attrName_, value_);
                }
              }
            }
          }
        }
      }
      if ( topComments_.length() > 0 ) {
        map_.put("topComments", topComments_);
      }
      if ( bottomComments_.length() > 0 ) {
        map_.put("bottomComments", bottomComments_);
      }
    }
    return map_;
  }

  // 29.11.08: WA on internal NPE when value is null:
  public static String getNodeValueAsString(Node node) {
    String ret_ = null;
    try {
      ret_ = node.getNodeValue();
      if ( ret_ != null ) {
        ret_ = ret_.trim();
        if ( "".equals(ret_) ) ret_ = null;
      }
    } catch (Exception x) {}
    return ret_;
  }

  public static boolean isLeafNode(Node node) {
    boolean ret_ = false;
    if ( node.hasChildNodes() ) {
      Node node_ = node.getFirstChild();
      if ( node_ != null && node_.getNodeType() == Node.TEXT_NODE ) {
        ret_ = true;
      }
    }
    return ret_;
  }

  public static String getComments(Map map, String mode) {
    String ret_ = null;
    if ( map != null ) {
      if ( "top".equals(mode) ) {
        ret_ = (String)map.get("topComments");
      } else
      if ( "bottom".equals(mode) ) {
        ret_ = (String)map.get("bottomComments");
      } else {
        ret_ = "Util.getComments(): Unexpected mode: \""+mode+"\".";
      }
    }
    return ret_;
  }

  public static String getSholdPresentation(Node node) {
    String ret_ = "";
    while ( true ) {
      Map map_ = getNodeChildValues(node);
      if ( map_ == null ) break;

      ret_ += ">>> ";
      String date2_      = (String)map_.get("row.timestamp");
      if ( date2_ != null ) ret_ += "[as of "+date2_+"] ";

      String topComments_ = (String)map_.get("topComments");
      if ( topComments_ != null ) {
        ret_ += topComments_; // ends with LF
      }

      String isPresence_ = (String)map_.get("is.presence");
      if ( "false".equals(isPresence_) ) {
        ret_ += "none"+LF;
      } else {
        String name_       = (String)map_.get("name");
        String origin_     = (String)map_.get("origin.name");
        String shareProc_  = (String)map_.get("share.procent");
        String shareValue_ = (String)map_.get("value");
        String currcy_     = (String)map_.get("currency");
        String isUltimate_ = (String)map_.get("is.share.ultimate");
        String isEstim_    = (String)map_.get("is.share.est");

        ret_ += ( name_ != null ? name_ : "<n/a>" );
        if ( origin_ != null || shareProc_ != null || shareValue_ != null || isUltimate_ != null ) {
          ret_ += " (";
          if ( origin_     != null ) ret_ += origin_+"; ";

          String str_ = "";
          if ( shareProc_  != null ) str_ += shareProc_+"%, ";
          if ( shareValue_ != null ) str_ += shareValue_+" "+currcy_+", ";
          if ( "true".equals(isUltimate_) ) str_ += "ultimate, ";
          if ( str_.length() > 0 ) {
            ret_ += "share: ";
            ret_ += str_.substring(0, str_.lastIndexOf(", "));
            if ( "true".equals(isEstim_) ) ret_ += " [E]";
          }
          ret_ += ")"+LF;
        }
      }

      String bottomComments_ = (String)map_.get("bottomComments");
      if ( bottomComments_ != null ) {
        ret_ += bottomComments_; // ends with LF
      }

      break;
    }
    return ( ret_.length() > 0 ? ret_ : null );
  }

}
